@@ -32,6 +32,7 @@ import java.net.Socket;
 import java.net.SocketAddress;
 import java.nio.ByteBuffer;
 import java.nio.channels.ByteChannel;
+import java.nio.channels.CancelledKeyException;
 import java.nio.channels.ClosedChannelException;
 import java.nio.channels.SelectionKey;
 
@@ -44,8 +45,8 @@ import javax.net.ssl.SSLException;
 import javax.net.ssl.SSLSession;
 
 import org.apache.http.HttpHost;
-import org.apache.http.annotation.ThreadingBehavior;
 import org.apache.http.annotation.Contract;
+import org.apache.http.annotation.ThreadingBehavior;
 import org.apache.http.nio.reactor.EventMask;
 import org.apache.http.nio.reactor.IOSession;
 import org.apache.http.nio.reactor.SessionBufferStatus;
@@ -61,10 +62,10 @@ import org.apache.http.util.Asserts;
  * The resultant instance of {@code SSLIOSession} must be added to the original
  * I/O session as an attribute with the {@link #SESSION_KEY} key.
  * <pre>
- *  SSLContext sslcontext = SSLContext.getInstance("SSL");
- *  sslcontext.init(null, null, null);
+ *  SSLContext sslContext = SSLContext.getInstance("SSL");
+ *  sslContext.init(null, null, null);
  *  SSLIOSession sslsession = new SSLIOSession(
- *      iosession, SSLMode.CLIENT, sslcontext, null);
+ *      iosession, SSLMode.CLIENT, sslContext, null);
  *  iosession.setAttribute(SSLIOSession.SESSION_KEY, sslsession);
  * </pre>
  *
@@ -86,7 +87,6 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
     private final SSLBuffer inEncrypted;
     private final SSLBuffer outEncrypted;
     private final SSLBuffer inPlain;
-    private final SSLBuffer outPlain;
     private final InternalByteChannel channel;
     private final SSLSetupHandler handler;
 
@@ -163,7 +163,6 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
         // Allocate buffers for application (unencrypted) data
         final int appBuffersize = this.sslEngine.getSession().getApplicationBufferSize();
         this.inPlain = bufferManagementStrategy.constructBuffer(appBuffersize);
-        this.outPlain = bufferManagementStrategy.constructBuffer(appBuffersize);
     }
 
     /**
@@ -238,7 +237,6 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
         this.inEncrypted.release();
         this.outEncrypted.release();
         this.inPlain.release();
-        this.outPlain.release();
 
         doHandshake();
     }
@@ -294,34 +292,24 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
         while (handshaking) {
             switch (this.sslEngine.getHandshakeStatus()) {
             case NEED_WRAP:
-                // Generate outgoing handshake data
+               // Generate outgoing handshake data
 
-                // Acquire buffers
-                ByteBuffer outPlainBuf = this.outPlain.acquire();
-                final ByteBuffer outEncryptedBuf = this.outEncrypted.acquire();
+               // Acquire buffer
+               final ByteBuffer outEncryptedBuf = this.outEncrypted.acquire();
 
-                // Perform operations
-                outPlainBuf.flip();
-                result = doWrap(outPlainBuf, outEncryptedBuf);
-                outPlainBuf.compact();
+               // Just wrap an empty buffer because there is no data to write.
+               result = doWrap(ByteBuffer.allocate(0), outEncryptedBuf);
 
-                // Release outPlain if empty
-                if (outPlainBuf.position() == 0) {
-                    this.outPlain.release();
-                    outPlainBuf = null;
-                }
-
-
-                if (result.getStatus() != Status.OK) {
-                    handshaking = false;
-                }
-                break;
+               if (result.getStatus() != Status.OK) {
+                   handshaking = false;
+               }
+               break;
             case NEED_UNWRAP:
                 // Process incoming handshake data
 
                 // Acquire buffers
-                ByteBuffer inEncryptedBuf = this.inEncrypted.acquire();
-                ByteBuffer inPlainBuf = this.inPlain.acquire();
+                final ByteBuffer inEncryptedBuf = this.inEncrypted.acquire();
+                final ByteBuffer inPlainBuf = this.inPlain.acquire();
 
                 // Perform operations
                 inEncryptedBuf.flip();
@@ -337,13 +325,11 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
                     // Release inEncrypted if empty
                     if (inEncryptedBuf.position() == 0) {
                         this.inEncrypted.release();
-                        inEncryptedBuf = null;
                     }
                 }
 
                 if (this.status >= IOSession.CLOSING) {
                     this.inPlain.release();
-                    inPlainBuf = null;
                 }
                 if (result.getStatus() != Status.OK) {
                     handshaking = false;
@@ -372,6 +358,9 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
 
     private void updateEventMask() {
         // Graceful session termination
+        if (this.status == CLOSING && !this.outEncrypted.hasData()) {
+            this.sslEngine.closeOutbound();
+        }
         if (this.status == CLOSING && this.sslEngine.isOutboundDone()
                 && (this.endOfStream || this.sslEngine.isInboundDone())) {
             this.status = CLOSED;
@@ -471,19 +460,20 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
 
             try {
                 if (!inEncryptedBuf.hasRemaining() && result.getHandshakeStatus() == HandshakeStatus.NEED_UNWRAP) {
-                    throw new SSLException("Input buffer is full");
+                    throw new SSLException("Unable to complete SSL handshake");
                 }
-                if (result.getStatus() == Status.OK) {
+                final Status status = result.getStatus();
+                if (status == Status.OK) {
                     decrypted = true;
                 } else {
+                    if (status == Status.BUFFER_UNDERFLOW && this.endOfStream) {
+                        throw new SSLException("Unable to decrypt incoming data due to unexpected end of stream");
+                    }
                     break;
                 }
                 if (result.getHandshakeStatus() != HandshakeStatus.NOT_HANDSHAKING) {
                     break;
                 }
-                if (this.endOfStream) {
-                    break;
-                }
             } finally {
                 // Release inEncrypted if empty
                 if (this.inEncrypted.acquire().position() == 0) {
@@ -570,39 +560,19 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
         if (this.status != ACTIVE) {
             throw new ClosedChannelException();
         }
-        if (this.outPlain.hasData()) {
-            // Acquire buffers
-            ByteBuffer outPlainBuf = this.outPlain.acquire();
-            final ByteBuffer outEncryptedBuf = this.outEncrypted.acquire();
-
-            // Perform operations
-            outPlainBuf.flip();
-            doWrap(outPlainBuf, outEncryptedBuf);
-            outPlainBuf.compact();
-
-            // Release outPlain if empty
-            if (outPlainBuf.position() == 0) {
-                this.outPlain.release();
-                outPlainBuf = null;
-            }
-        }
-        if (!this.outPlain.hasData()) {
-            final ByteBuffer outEncryptedBuf = this.outEncrypted.acquire();
-            final SSLEngineResult result = doWrap(src, outEncryptedBuf);
-            if (result.getStatus() == Status.CLOSED) {
-                this.status = CLOSED;
-            }
-            return result.bytesConsumed();
-        } else {
-            return 0;
+        final ByteBuffer outEncryptedBuf = this.outEncrypted.acquire();
+        final SSLEngineResult result = doWrap(src, outEncryptedBuf);
+        if (result.getStatus() == Status.CLOSED) {
+           this.status = CLOSED;
         }
+        return result.bytesConsumed();
     }
 
     private synchronized int readPlain(final ByteBuffer dst) {
         Args.notNull(dst, "Byte buffer");
         if (this.inPlain.hasData()) {
             // Acquire buffer
-            ByteBuffer inPlainBuf = this.inPlain.acquire();
+            final ByteBuffer inPlainBuf = this.inPlain.acquire();
 
             // Perform opertaions
             inPlainBuf.flip();
@@ -615,7 +585,6 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
             // Release if empty
             if (inPlainBuf.position() == 0) {
                 this.inPlain.release();
-                inPlainBuf = null;
             }
             return n;
         } else {
@@ -632,12 +601,16 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
         if (this.status >= CLOSING) {
             return;
         }
-
-        // deactivating buffers in here causes failed tests
-
         this.status = CLOSING;
+        if (this.session.getSocketTimeout() == 0) {
+            this.session.setSocketTimeout(1000);
+        }
         this.sslEngine.closeOutbound();
-        updateEventMask();
+        try {
+            updateEventMask();
+        } catch (final CancelledKeyException ex) {
+            shutdown();
+        }
     }
 
     @Override
@@ -645,14 +618,13 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
         if (this.status == CLOSED) {
             return;
         }
+        this.status = CLOSED;
+        this.session.shutdown();
 
         this.inEncrypted.release();
         this.outEncrypted.release();
         this.inPlain.release();
-        this.outPlain.release();
 
-        this.status = CLOSED;
-        this.session.shutdown();
     }
 
     @Override
@@ -723,8 +695,7 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
     @Override
     public synchronized boolean hasBufferedOutput() {
         return (this.appBufferStatus != null && this.appBufferStatus.hasBufferedOutput())
-            || this.outEncrypted.hasData()
-            || this.outPlain.hasData();
+            || this.outEncrypted.hasData();
     }
 
     @Override
@@ -791,8 +762,6 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
         buffer.append(!this.inPlain.hasData() ? 0 : inPlain.acquire().position());
         buffer.append("][");
         buffer.append(!this.outEncrypted.hasData() ? 0 : outEncrypted.acquire().position());
-        buffer.append("][");
-        buffer.append(!this.outPlain.hasData() ? 0 : outPlain.acquire().position());
         buffer.append("]");
         return buffer.toString();
     }
